@@ -1,5 +1,5 @@
 /*
- *  $Id: ui_getc.c,v 1.62 2011/06/24 23:49:42 tom Exp $
+ *  $Id: ui_getc.c,v 1.65 2011/10/20 23:45:48 tom Exp $
  *
  *  ui_getc.c - user interface glue for getc()
  *
@@ -294,8 +294,7 @@ valid_file(FILE *fp)
     int fd = fileno(fp);
 
     if (fd >= 0) {
-	long result = 0;
-	if ((result = fcntl(fd, F_GETFL, 0)) >= 0) {
+	if (fcntl(fd, F_GETFL, 0) >= 0) {
 	    code = TRUE;
 	}
     }
@@ -415,6 +414,8 @@ dlg_getc(WINDOW *win, int *fkey)
 	wtimeout(win, interval);
 
     while (!done) {
+	bool handle_others = FALSE;
+
 	/*
 	 * If there was no pending file-input, check the keyboard.
 	 */
@@ -513,15 +514,7 @@ dlg_getc(WINDOW *win, int *fkey)
 		    break;
 		}
 #endif
-		if ((p = dialog_state.getc_redirect) != 0) {
-		    if (!(p->handle_getc(p, ch, *fkey, &result))) {
-			dlg_remove_callback(p);
-			dialog_state.getc_redirect = 0;
-			win = save_win;
-		    }
-		} else {
-		    done = TRUE;
-		}
+		handle_others = TRUE;
 		break;
 #ifdef HAVE_DLG_TRACE
 	    case CHR_TRACE:
@@ -529,6 +522,21 @@ dlg_getc(WINDOW *win, int *fkey)
 		break;
 #endif
 	    }
+	} else {
+	    handle_others = TRUE;
+	}
+
+	if (handle_others) {
+	    if ((p = dialog_state.getc_redirect) != 0) {
+		if (!(p->handle_getc(p, ch, *fkey, &result))) {
+		    done = (p->win == save_win) && (!p->keep_win);
+		    dlg_remove_callback(p);
+		    dialog_state.getc_redirect = 0;
+		    win = save_win;
+		}
+	    } else {
+		done = TRUE;
+	    }
 	}
     }
     if (literal)
